library seekbar;

import 'package:flutter/material.dart';

/// A seekBar widget for audio/video players
class SeekBar extends StatefulWidget {
  final double currentTime;
  final double durationTime;

  final double progressBarHeight;
  final double progressLength;
  final double progressRadius;
  final double progressFontSize;

  final Color barColor;
  final Color progressColor;
  final Color progressTextColor;

  final Function onStartTrackingTouch;
  final ValueChanged<double> onProgressChanged;
  final Function onStopTrackingTouch;

  SeekBar({
    Key key,
    this.currentTime = 0.0,
    this.durationTime = 0.0,

    this.progressBarHeight = 3.0,
    this.progressLength = 120.0,
    this.progressRadius = 12.0,
    this.progressFontSize = 11.0,

    this.barColor = const Color(0x73FFFFFF),
    this.progressColor = const Color(0xFF79A5F2),
    this.progressTextColor = Colors.white,

    this.onStartTrackingTouch,
    this.onProgressChanged,
    this.onStopTrackingTouch,
  }) : super(key: key);

  @override
  _SeekBarState createState() {
    return _SeekBarState();
  }
}

class _SeekBarState extends State<SeekBar> {
  Offset _touchPoint = Offset.zero;

  double _value = 0.0;
  double _currentTime = 0.0;

  bool _touchDown = false;

  _setValue() {
    _value = _touchPoint.dx <= widget.progressLength / 2 ? 0.0 : _touchPoint.dx >= context.size.width - widget.progressLength / 2 ? 1.0 : (_touchPoint.dx - widget.progressLength / 2) / (context.size.width - widget.progressLength);
    _currentTime = _value * widget.durationTime;
  }

  _checkTouchPoint() {
    if (_touchPoint.dx <= 0) {
      _touchPoint = Offset(0, _touchPoint.dy);
    }
    if (_touchPoint.dx >= context.size.width) {
      _touchPoint = Offset(context.size.width, _touchPoint.dy);
    }
  }

  @override
  void initState() {
    _value = widget.currentTime > widget.durationTime ? 1 : widget.durationTime <= 0 ? 0 : widget.currentTime / widget.durationTime;
    _currentTime = widget.currentTime;
    
    super.initState();
  }

  @override
  void didUpdateWidget(SeekBar oldWidget) {
    _value = widget.currentTime > widget.durationTime ? 1 : widget.durationTime <= 0 ? 0 : widget.currentTime / widget.durationTime;
    _currentTime = widget.currentTime;

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onHorizontalDragDown: (details) {
        RenderBox box = context.findRenderObject();
        _touchPoint = box.globalToLocal(details.globalPosition);
        _checkTouchPoint();
        setState(() {
          _setValue();
          _touchDown = true;
        });
        if (widget.onStartTrackingTouch != null) {
          widget.onStartTrackingTouch();
        }
      },
      onHorizontalDragUpdate: (details) {
        RenderBox box = context.findRenderObject();
        _touchPoint = box.globalToLocal(details.globalPosition);
        _checkTouchPoint();
        setState(() {
          _setValue();
        });
        if (widget.onProgressChanged != null) {
          widget.onProgressChanged(_value);
        }
      },
      onHorizontalDragEnd: (details) {
        setState(() {
          _touchDown = false;
        });
        if (widget.onStopTrackingTouch != null) {
          widget.onStopTrackingTouch();
        }
      },
      child: Container(
        constraints: BoxConstraints.expand(height: widget.progressRadius * 2),
        child: CustomPaint(
          painter: _SeekBarPainter(
            progressBarHeight: widget.progressBarHeight,
            progressLength: widget.progressLength,
            progressRadius: widget.progressRadius,
            progressFontSize: widget.progressFontSize,
            value: _value,
            currentTime: _currentTime,
            durationTime: widget.durationTime,
            barColor: widget.barColor,
            progressColor: widget.progressColor,
            progressTextColor: widget.progressTextColor,
            touchDown: _touchDown,
          ),
        ),
      ),
    );
  }
}

class _SeekBarPainter extends CustomPainter {
  final double value;
  final double currentTime;
  final double durationTime;
  
  final double progressBarHeight;
  final double progressLength;
  final double progressRadius;
  final double progressFontSize;
  
  final Color barColor;
  final Color progressColor;
  final Color progressTextColor;

  final bool touchDown;

  _SeekBarPainter({
    this.progressBarHeight,
    this.progressLength,
    this.progressRadius,
    this.progressFontSize,
    this.value,
    this.currentTime,
    this.durationTime,
    this.barColor,
    this.progressColor,
    this.progressTextColor,
    this.touchDown,
  });

  @override
  bool shouldRepaint(_SeekBarPainter old) {
    return value != old.value ||
        currentTime != old.currentTime ||
        durationTime != old.durationTime ||
        touchDown != old.touchDown;
  }

  String buildTimeStringFromDouble(double t) {
    int hours = (t / 3600).truncate();
    int minutes = ((t - hours.toDouble() * 3600) / 60).truncate();
    int seconds = (t - hours.toDouble() * 3600 - minutes.toDouble() * 60).truncate();

    return (hours > 0 ? hours.toString().padLeft(2, '0') + ":" : "") + minutes.toString().padLeft(2, '0') + ":" + seconds.toString().padLeft(2, '0');
  }

  @override
  void paint(Canvas canvas, Size size) {
    final Paint paint = Paint()
      ..isAntiAlias = true
      ..strokeCap = StrokeCap.square
      ..strokeWidth = progressBarHeight;

    final centerY = size.height / 2.0;
    final barLength = size.width;

    final Offset startPoint = Offset(2.0, centerY);
    final Offset endPoint = Offset(size.width - 2.0, centerY);

    final Offset progressPoint = Offset((barLength - progressLength) * value + (progressLength / 2), centerY);

    paint.color = barColor;
    canvas.drawLine(startPoint, endPoint, paint);

    paint.color = progressColor;
    canvas.drawLine(startPoint, progressPoint, paint);

    canvas.drawRRect(
      RRect.fromLTRBXY(
        (barLength - progressLength) * value, 
        centerY - progressRadius, 
        (barLength - progressLength) * value + progressLength, 
        centerY + progressRadius, 
        progressRadius, 
        progressRadius), paint);

    final TextSpan span = new TextSpan(
      style: new TextStyle(color: progressTextColor, fontSize: progressFontSize), 
      text: buildTimeStringFromDouble(currentTime) + " / " + buildTimeStringFromDouble(durationTime)
    );
    
    final TextPainter tp = new TextPainter(text: span, textAlign: TextAlign.center, textDirection: TextDirection.ltr);
    tp.layout(minWidth: progressLength);
    tp.paint(canvas, new Offset((barLength - progressLength) * value, centerY - progressFontSize / 2));
  }
}